#========================================================================================
# package      DE_EPAGES::Customer::t::unit::CustomerPatch
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for DE_EPAGES::Customer::API::CustomerPatch.
#========================================================================================
package DE_EPAGES::Customer::t::unit::CustomerPatch;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use DE_EPAGES::Test::Mock::Mockify qw ( GetParametersFromMockifyCall );
use DE_EPAGES::Test::Mock::Mockery;
use Test::Exception;
use Test::More;

use strict;



sub test {
    my $self = shift;

    $self->test_add_customerNumber();
    $self->test_add_internalNote();
    $self->test_add_billingAddress();

    $self->test_remove_customerNumber();
    $self->test_remove_internalNote();
    $self->test_remove_billingAddress();

    done_testing();
}



sub test_add_customerNumber {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Mockery = $self->_createMocks();
    my $CustomerPatch = $Mockery->get('SUT');

    my $hPatchOperation = {
        'op' => 'add',
        'path' => '/customerNumber',
        'value' => '123',
    };

    $CustomerPatch->patch($hPatchOperation);

    my $Customer = $Mockery->get('Customer');
    is($Customer->alias(), '123', "$SubTestName - test if the attribute customerNumber was set.");
}



sub test_add_internalNote {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Mockery = $self->_createMocks();
    my $CustomerPatch = $Mockery->get('SUT');

    my $hPatchOperation = {
        'op' => 'add',
        'path' => '/internalNote',
        'value' => 'internal test node',
    };

    $CustomerPatch->patch($hPatchOperation);

    my $Customer = $Mockery->get('Customer');
    is($Customer->get('Comment'), 'internal test node', "$SubTestName - test if the attribute internalNote was set.");
}



sub test_add_billingAddress {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Mockery = $self->_createMocks();
    my $CustomerPatch = $Mockery->get('SUT');

    my $hPatchOperation = {
        'op' => 'add',
        'path' => '/billingAddress',
        'value' => {
            'company'       => 'Test GmbH',
            'firstName'     => 'Joe',
            'lastName'      => 'Tester',
            'street'        => '123 Test St.',
            'streetDetails' => 'end of the street',
            'zipCode'       => '12345',
            'city'          => 'Testburg',
            'state'         => 'TST',
            'title'         => 'Test Engineer',
            'emailAddress'  => 'me@testing.org',
            'vatId'         => 'vat soll das',
            'salutation'    => 'Salve!',
        },
    };

    $CustomerPatch->patch($hPatchOperation);

    my $SoapOrderService = $Mockery->get('SoapCustomerService');
    my $aSetAddressArguments = GetParametersFromMockifyCall($SoapOrderService, 'setAddress');

    my $hAddressData = $aSetAddressArguments->[1];
    my $hExpectedAddressData = $self->_getExpectedAddressData();
    is_deeply($hAddressData, $hExpectedAddressData, "$SubTestName - test if the expected address data was used.");
    my $AddressType = $aSetAddressArguments->[2];
    is($AddressType, 'BillingAddress', "$SubTestName - test if the address type was billing.");
}



sub test_remove_customerNumber {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Mockery = $self->_createMocks();
    my $CustomerPatch = $Mockery->get('SUT');

    my $hPatchOperation = {
        'op' => 'remove',
        'path' => '/customerNumber',
    };

    throws_ok(sub {
        $CustomerPatch->patch($hPatchOperation);
    }, qr/BadRequest: Reason=Removing \/customerNumber is not supported!/,
    "$SubTestName - test if the attribute customerNumber cannot be removed.");
}



sub test_remove_internalNote {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Mockery = $self->_createMocks();
    my $CustomerPatch = $Mockery->get('SUT');

    my $hPatchOperation = {
        'op' => 'remove',
        'path' => '/internalNote',
    };

    $CustomerPatch->patch($hPatchOperation);

    my $Customer = $Mockery->get('Customer');
    is($Customer->get('Comment'), undef, "$SubTestName - test if the attribute internalNote was removed.");
}



sub test_remove_billingAddress {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Mockery = $self->_createMocks();
    my $CustomerPatch = $Mockery->get('SUT');

    my $hPatchOperation = {
        'op' => 'remove',
        'path' => '/billingAddress',
    };

    throws_ok(sub {
        $CustomerPatch->patch($hPatchOperation);
    }, qr/BadRequest: Reason=Removing \/billingAddress is not supported!/,
    "$SubTestName - test if the attribute billingAddress cannot be removed.");
}



sub _createMocks {
    my $self = shift;

    my $Mockery = DE_EPAGES::Test::Mock::Mockery->new();

    $Mockery->register('SUT', ['Customer', 'SoapCustomerService'], sub {
        my ($Customer, $SoapCustomerService) = @_;
        my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::Customer::API::CustomerPatch', [$Customer]);
        $Mockify->addMockWithReturnValue('_createSoapCustomerService', $SoapCustomerService);
        return $Mockify->getMockObject();
    });

    $Mockery->register('Customer', ['Address'], sub {
        my ($Address) = @_;
        my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Customer::API::Object::Customer', {
            'Alias' => 'Product0001',
            'BillingAddress' => $Address,
        });
        return $Mockify->getMockObject();
    });

    $Mockery->register('SoapCustomerService', [], sub {
        my $hOptions = {'Object' => 1};
        my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::Customer::API::WebService::CustomerService', [%$hOptions]);
        $Mockify->addMockWithReturnValueAndParameterCheck('setAddress', undef, ['object', 'hashref', 'string']);
        return $Mockify->getMockObject();
    });

    $Mockery->register('Address', [], sub {
        my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Address::API::Object::Address', {
            'ID' => '12345',
            'Company'=> 'billing company',
            'Salutation'=> undef,
            'FirstName'=> 'fname',
            'LastName'=> 'lname',
            'Street'=> 'billstr 1',
            'Street2'=> undef,
            'Zipcode'=> '100001',
            'City'=> 'onetown',
            'State'=> undef,
            'CountryID'=> undef,
            'Title'=> undef,
            'VATID'=> undef,
            'Birthday'=> undef,
            'EMail'=> 'first.last@email.com',
        });
        $Mockify->addMockWithReturnValue('delete', undef);
        return $Mockify->getMockObject();
    });

    return $Mockery->assemble();
}



sub _getExpectedAddressData {
    return {
        'Street2' => 'end of the street',
        'FirstName' => 'Joe',
        'Street' => '123 Test St.',
        'City' => 'Testburg',
        'State' => 'TST',
        'Salutation' => 'Salve!',
        'Title' => 'Test Engineer',
        'CountryID' => undef,
        'Zipcode' => '12345',
        'VATID' => 'vat soll das',
        'LastName' => 'Tester',
        'Birthday' => undef,
        'EMail' => 'me@testing.org',
        'Company' => 'Test GmbH',
    };
}

__PACKAGE__->runScript();
